package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.*;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.aspect.MonitorLog;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesSortShippingCheckModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesJisShipping;
import cn.estsh.i3plus.pojo.mes.bean.MesPartShippingGroup;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingList;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingListDetail;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagementDetail;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-04-26 15:38
 * @Modify:
 **/
@Service
@Slf4j
public class MesSortShippingCheckService implements IMesSortShippingCheckService {

    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;
    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderManagementDetailRepository;
    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Autowired
    private MesPartShippingGroupRepository partShippingGroupRepository;
    @Autowired
    private MesLoadingListRepository listRepository;

    @Autowired
    private MesLoadingListDetailRepository listDetailRepository;

    @Autowired
    private IMesShippingLoadingCheckService shippingLoadingCheckService;

    @Autowired
    private MesJisShippingRepository jisShippingRepository;

    @Autowired
    private IMesJisShippingService jisShippingService;
    @Autowired
    private MesLoadingListDetailRepository vehiclesOrderDetailRepository;
    @Autowired
    private MesLoadingListRepository vehiclesOrderRepository;

    @Autowired
    private MesCustomerPartRepository mesCustomerPartRepository;

    @Autowired
    private MesPartShippingGroupService mesPartShippingGroupService;

    @Autowired
    private IMesConfigService mesConfigService;

    @Override
    public MesSortShippingCheckModel doShippingOrderNoQuery(MesShippingOrderManagement shippingOrderManagement) {
        //校验发运单是否存在
        DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(shippingOrderManagement.getShippingCode(), "shippingCode", orderManagementPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderManagementPackBean);
        if (StringUtils.isEmpty(orderManagement)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在，请检查数据！", shippingOrderManagement.getShippingCode()));
        }

        //校验发运单状态 必须是未装车条码
        if (orderManagement.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue() && orderManagement.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
            throw new ImppBusiException(String.format("【%s】此发运单状态为【%s】,请选择状态为【%s】或【%s】状态的发运单", shippingOrderManagement.getShippingCode(),
                    MesExtEnumUtil.SHIPPING_ORDER_STATUS.valueOfDescription(orderManagement.getStatus()),
                    MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getDescription(), MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getDescription()));
        }

        //校验发运单扫描顺序
        List<Integer> statusForCheckList = new ArrayList<>();
        statusForCheckList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue());
        statusForCheckList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue());
        statusForCheckList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue());
        statusForCheckList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.PRINTED.getValue());
        List<Integer> status = new ArrayList<>();
        status.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue());
        status.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.PRINTED.getValue());
        status.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue());
        status.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.LOADING.getValue());
        status.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.STATISTICSING.getValue());
        status.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.STATISTICS.getValue());
        //最后一个扫描完成的发运单信息
        orderManagementPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getNumberSmallerPack(orderManagement.getSeq(), "seq", orderManagementPackBean);
        DdlPreparedPack.getStringEqualPack(orderManagement.getShippingGroupCode(),"shippingGroupCode",orderManagementPackBean);
        DdlPreparedPack.getInPackList(status, "status", orderManagementPackBean);
        DdlPreparedPack.getOrderBy("seq",CommonEnumUtil.ASC_OR_DESC.DESC.getValue(),orderManagementPackBean);
        MesShippingOrderManagement management = shippingOrderManagementRepository.getByProperty(orderManagementPackBean);
        //是否存在
        orderManagementPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getNumberSmallerPack(orderManagement.getSeq(), "seq", orderManagementPackBean);
        if(!Objects.isNull(management)){
            DdlPreparedPack.getNumberBiggerPack(management.getSeq(), "seq", orderManagementPackBean);
        }
        DdlPreparedPack.getStringEqualPack(orderManagement.getShippingGroupCode(),"shippingGroupCode",orderManagementPackBean);
        DdlPreparedPack.getInPackList(statusForCheckList, "status", orderManagementPackBean);
        boolean exists = shippingOrderManagementRepository.isExitByHql(orderManagementPackBean);
        if (exists) {
            if((!Objects.isNull(shippingOrderManagement.getIsPass()) && CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == shippingOrderManagement.getIsPass())){
                //updateLoadingListClose(statusForCheckList,orderManagement,shippingOrderManagement.getModifyUser());
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(orderManagement.getOrganizeCode());
                DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "id",packBean);
                shippingOrderManagementRepository.updateByProperties(new String[]{"forceFlag","modifyUser", "modifyDatetime","remark"},
                        new Object[]{MesPcnExtConstWords.STRONGER_PASS,shippingOrderManagement.getModifyUser(), TimeTool.getNowTime(true),MesPcnExtConstWords.STRONGER_PASS}, packBean);
            }else{
                throw new ImppBusiException(String.format("【%s】发运单对应排序【%s】前存在未扫描发运的发运单，请检查数据！",
                        orderManagement.getShippingCode(), orderManagement.getSeq()));
            }
        }
        //根据发运单查询装车单明细信息
        boolean isUpdate = false;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(shippingOrderManagement.getShippingCode(),"shippingCode",packBean);
        MesLoadingListDetail listDetail = listDetailRepository.getByProperty(packBean);
        if(!Objects.isNull(listDetail)){
            packBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(listDetail.getPid(), "id", packBean);
            MesLoadingList loadingList = listRepository.getByProperty(packBean);
            if(!Objects.isNull(loadingList)){
                orderManagement.setOrderCode(loadingList.getOrderCode());
                orderManagement.setLoadingListId(loadingList.getId());
                isUpdate = true;
            }
        }


        //根据扫描的发运单找到是否装车配置，若是，则根据发运单查询是否生成装车单，若没生成装车单则提示此发运单配置是否装车为是,但没生成装车单，请检查数据！
        if (Objects.equals(orderManagement.getIsLoading(), MesCommonConstant.TRUE_INTEGER)) {

            if (StringUtils.isEmpty(orderManagement.getOrderCode()) ) {
                throw new ImppBusiException(String.format("【%s】此发运单配置是否装车为是,但没绑定装车单号，请检查数据！", shippingOrderManagement.getShippingCode()));
            }


            DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(orderManagement.getOrderCode(), "orderCode", orderDetailPackBean);
            MesLoadingList loadingList = listRepository.getByProperty(orderDetailPackBean);
            if (StringUtils.isEmpty(loadingList)) {
                throw new ImppBusiException(String.format("【%s】此发运单配置是否装车为是,但没生成装车单，请检查数据！", shippingOrderManagement.getShippingCode()));
            }


            //判断是否先装车
            if (Objects.equals(orderManagement.getIsFirstInstall(), MesCommonConstant.TRUE_INTEGER)) {

                //若没进行装车校验，则提示【%s】此发运单对应的【%s】此装车单状态为【%s】，请选择状态为【%s】的数据
                if (loadingList.getStatus() != MesExtEnumUtil.MES_LOADING_STATUS.SCANNED.getValue()) {
                    throw new ImppBusiException(String.format("【%s】此发运单对应的【%s】此装车单状态为【%s】，请选择状态为【%s】的数据！",
                            shippingOrderManagement.getShippingCode(), loadingList.getOrderCode(),
                            MesExtEnumUtil.MES_LOADING_STATUS.valueOfDescription(loadingList.getStatus()),
                            MesExtEnumUtil.MES_LOADING_STATUS.SCANNED.getDescription()));
                }
            }
        }

        MesSortShippingCheckModel model = new MesSortShippingCheckModel();
        if (!StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {
            model.setIsScanLocationCode(true);
        }


        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "pid", orderDetailPackBean);
        DdlPreparedPack.getNumberSmallerPack(MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.CLOSE.getValue(),"status",orderDetailPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"custInfoSeq"}, orderDetailPackBean);
        List<MesShippingOrderManagementDetail> detailList = shippingOrderManagementDetailRepository.findByHqlWhere(orderDetailPackBean);

        detailList.forEach(k -> {
            if (k.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue() || k.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SKIP.getValue()){
                k.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);
            }
        });

        Long count = detailList.stream().filter(k -> Objects.equals(k.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)).count();
        model.setIsScanCount(count.intValue());

        //查询最近扫描完成的装车单号
        DdlPackBean recentlyOrderPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue(), "status", recentlyOrderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"modifyDatetime"}, recentlyOrderPackBean);
        MesShippingOrderManagement recentlyVehiclesOrder = shippingOrderManagementRepository.getByProperty(recentlyOrderPackBean);

        model.setDetailList(detailList);
        model.setShippingCode(orderManagement.getShippingCode());
        if (!StringUtils.isEmpty(recentlyVehiclesOrder)) {

            model.setRecentlyScanShippingCode(recentlyVehiclesOrder.getShippingCode());
        }
        //发运单关联装车单字段赋值
        if(isUpdate){
            ConvertBean.serviceModelUpdate(orderManagement,shippingOrderManagement.getModifyUser());
            update(orderManagement);
        }

        return model;
    }

    @Override
    public MesSortShippingCheckModel saveSn(MesSortShippingCheckModel model, String organizeCode, String userInfo) {
        //校验发运单是否存在
        MesShippingOrderManagement orderManagement = checkMesShippingOrderManagement(model, organizeCode);
        //零件发运组
        MesPartShippingGroup shippingGroup = getMesPartShippingGroup(organizeCode, orderManagement.getShippingGroupCode());
        //根据条码找到对应的物料号，若物料号一致，则修改扫描flg，将flg改为true.
        //根据扫描的发运单以及前端传的发运单明细，进行匹配，若没有找到扫描的发运单，则提示扫描的发运单号不在此装车单中，请检查数据！。
        if(!Objects.isNull(model.getIsPass()) && CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == model.getIsPass()){
            Optional<MesShippingOrderManagementDetail> shippingOrderManagementDetailOptional = model.getDetailList().stream().filter(t -> !Objects.equals(t.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)).findFirst();
            if(shippingOrderManagementDetailOptional.isPresent()){
                MesShippingOrderManagementDetail mesShippingOrderManagementDetail = shippingOrderManagementDetailOptional.get();
                MesProduceSn produceSn = new MesProduceSn();
                produceSn.setProductSn(model.getSn());
                updateOrderManagementDetails(userInfo, produceSn, orderManagement, mesShippingOrderManagementDetail,MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SKIP.getValue(),MesPcnExtConstWords.SHIP_PASS);
            }else{
                throw new ImppBusiException(String.format("【%s】明细已全部扫描完成，请检查数据！", orderManagement.getShippingCode()));
            }
        }else{
            //校验条码
            MesProduceSn produceSn = checkSn(model, organizeCode);
            boolean scanFlg = false;
            for (MesShippingOrderManagementDetail detail : model.getDetailList()) {
                if (detail.getPartNo().equals(produceSn.getPartNo()) && StringUtils.isEmpty(detail.getBarcode()) && !Objects.equals(detail.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)) {
                    //校验发运单明细顺序
                    if(!StringUtils.isEmpty(shippingGroup.getIsEnableShippingFailSafe()) && CommonEnumUtil.VALID == shippingGroup.getIsEnableShippingFailSafe()){
                        Optional<MesShippingOrderManagementDetail> first = model.getDetailList().stream().filter(k -> k.getCustInfoSeq().compareTo(detail.getCustInfoSeq()) < 0 && !Objects.equals(k.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)).findFirst();
                        if(first.isPresent()){
                            throw new ImppBusiException(String.format("发运单【%s】启用明细顺序防错，序号【%s】前还有未扫描的发运单明细，请检查数据！", orderManagement.getShippingCode(), detail.getCustInfoSeq()));
                        }
                    }

                    //判断是否需要排序校验 默认排序则需要校验
                    if (!StringUtils.isEmpty(orderManagement.getScanSeqWay()) && orderManagement.getScanSeqWay() == MesExtEnumUtil.SHIPPING_GROUP_SCAN_CONFIRM_SEQ_MODE.DEFAULT_SORT.getValue()) {
                        //获取明细中最小的
                        Optional<String> minPartNO = model.getDetailList().stream().filter(k -> StringUtils.isEmpty(k.getBarcode()))
                                .min(Comparator.comparingLong(k -> k.getCustInfoSeq())).map(k -> k.getPartNo());
                        //若扫描的不是最小顺序 则报错
                        if (minPartNO.isPresent() && !minPartNO.get().equals(produceSn.getPartNo())) {
                            throw new ImppBusiException(String.format("【%s】此条码扫描顺序错误，请检查数据！", model.getSn()));
                        }
                    }
                    //校验客户零件信息是否存在
                    DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
                    DdlPreparedPack.getStringEqualPack(detail.getPartNo(), "erpPartNo", packBean);
                    DdlPreparedPack.getStringEqualPack(detail.getCustPartNo(), "custPartNo", packBean);
                    if(!mesCustomerPartRepository.isExitByHql(packBean)){
                        throw new ImppBusiException(String.format("零件号【%s】客户零件号【%s】关系不存在，请检查数据！", detail.getPartNo(),detail.getCustPartNo()));
                    }

                    scanFlg = true;
                    updateOrderManagementDetails(userInfo, produceSn, orderManagement, detail,MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue(),"扫描完成");
                    break;
                }
            }
            if (!scanFlg) {
                throw new ImppBusiException(String.format("【%s】此条码对应的【%s】物料号与发运单中物料号不匹配，请检查数据！", model.getSn(), produceSn.getPartNo()));
            }
        }


        Long count = model.getDetailList().stream().filter(k -> Objects.equals(k.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)).count();
        model.setIsScanCount(count.intValue());

        //当前发运单全部扫描完成 并且不需要扫位置码
        if (count.intValue() == model.getDetailList().size() && StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {

            //保存数据库
            saveDate(orderManagement, model, organizeCode, userInfo);
        }

        return model;
    }

    private void updateOrderManagementDetails(String userInfo, MesProduceSn produceSn, MesShippingOrderManagement orderManagement, MesShippingOrderManagementDetail detail,Integer status,String remark) {
        if(MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue() == status){
            detail.setBarcode(produceSn.getProductSn());
        }

        if (StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {
            detail.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);
            detail.setStatus(status);
            detail.setRemark(remark);
            //根据是否启动记忆判断 若启用记忆，则根据保存数据库
            if (!StringUtils.isEmpty(orderManagement.getIsEnableMemory()) && Objects.equals(orderManagement.getIsEnableMemory(), MesCommonConstant.TRUE_INTEGER)) {

                //若发运单状态为创建或发布 则写入开始扫描时间
                if (!StringUtils.isEmpty(orderManagement.getStatus()) && (orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() ||
                        orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue())) {
                    orderManagement.setStartScanTime(TimeTool.getNowTime(true));
                }

                orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue());
                orderManagement.setSystemSyncStatus(CommonEnumUtil.FALSE);
                ConvertBean.serviceModelUpdate(orderManagement, userInfo);
                update(orderManagement);

                detail.setActualQty(1);
                detail.setSystemSyncStatus(CommonEnumUtil.FALSE);
                ConvertBean.serviceModelUpdate(detail, userInfo);
                updateDetail(detail);
                //内部条码更新条码状态
                if (!Objects.isNull(produceSn.getId())) {
                    produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.SHIPPED.getValue());
                    produceSn.setSystemSyncStatus(CommonEnumUtil.FALSE);
                    ConvertBean.serviceModelUpdate(detail, userInfo);
                    produceSnExtService.update(produceSn);
                }
            }
        }
    }

    @Override
    public MesSortShippingCheckModel saveSnAndLocationCode(MesSortShippingCheckModel model, String organizeCode, String userInfo) {

        //校验条码
        MesProduceSn produceSn = checkMesProduceSn(model, organizeCode);


        //校验发运单是否存在
        MesShippingOrderManagement orderManagement = checkMesShippingOrderManagement(model, organizeCode);

        //根据条码找到对应的物料号，若物料号一致，则修改扫描flg，将flg改为true.
        boolean scanFlg = false;
        for (MesShippingOrderManagementDetail detail : model.getDetailList()) {
            if (detail.getPartNo().equals(produceSn.getPartNo())) {
                scanFlg = true;
                detail.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);
                detail.setBarcode(produceSn.getSerialNumber());
                break;
            }
        }

        if (!scanFlg) {
            throw new ImppBusiException(String.format("【%s】此条码对应的【%s】物料号与发运单中物料号不匹配，请检查数据！", model.getSn(), produceSn.getPartNo()));
        }

        Long count = model.getDetailList().stream().filter(k -> k.getIsScanFlg() == MesCommonConstant.TRUE_INTEGER).count();

        //当前发运单全部扫描完成
        if (count.intValue() == model.getDetailList().size()) {

            //保存数据库
            saveDate(orderManagement, model, organizeCode, userInfo);

        }

        return model;
    }

    @Override
    public MesShippingOrderManagement queryShippingOrderNoByAutoPushOrder(String shippingGroupNo, String org) {

        //查询最近扫描完成的装车单号
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue(), "status", orderPackBean);
//        DdlPreparedPack.getStringEqualPack(shippingGroupNo, "shippingGroupCode", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"seq"}, orderPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderPackBean);

        return orderManagement;

    }

    private MesPartShippingGroup getMesPartShippingGroup(String org, String partShippingGroupCode) {
        MesPartShippingGroup shippingGroup = mesPartShippingGroupService.getMesPartShippingGroup(org, partShippingGroupCode);
        if(Objects.isNull(shippingGroup)){
            throw new ImppBusiException(String.format("零件发运组【%s】信息不存在，请检查数据！", partShippingGroupCode));
        }
        return shippingGroup;
    }

    private MesProduceSn checkSn(MesSortShippingCheckModel model, String org) {
        //校验条码是否重复扫描
        Optional<MesShippingOrderManagementDetail> optional = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getBarcode()) && k.getBarcode().equals(model.getSn())).findFirst();
        if (optional.isPresent()) {
            throw new ImppBusiException(String.format("【%s】此条码已经扫描过，请检查数据！", model.getSn()));
        }
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getSn(), "productSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);
        if (!StringUtils.isEmpty(produceSn)) {
            //校验状态
            checkStatus(model, produceSn);
        }else{
            //外部条码解析
            MesShippingOrderManagementDetail mesShippingOrderManagementDetail = getMesShippingOrderManagementDetail(model, org);
            //封装信息
            produceSn = new MesProduceSn();
            produceSn.setSerialNumber(model.getSn());
            produceSn.setProductSn(model.getSn());
            produceSn.setPartNo(mesShippingOrderManagementDetail.getPartNo());
        }

        return produceSn;

    }

    private MesShippingOrderManagementDetail getMesShippingOrderManagementDetail(MesSortShippingCheckModel model, String org) {
        //外部解析条码长度卡控
        String cfgValue = mesConfigService.getCfgValue(org, MesPcnExtConstWords.MAX_SHIPPING_BARCODE_LENGTH);
        if(Integer.parseInt(cfgValue) < model.getSn().length()) MesPcnException.throwMesBusiException("【%s】此条码长度超过【%s】位，请检查数据！", model.getSn(),cfgValue);
        List<MesShippingOrderManagementDetail> detailList = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getCustPartNo()) && model.getSn().startsWith(k.getCustPartNo()) && StringUtils.isEmpty(k.getBarcode()) && !Objects.equals(k.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)).collect(Collectors.toList());
        MesShippingOrderManagementDetail mesShippingOrderManagementDetail = null;
        for (MesShippingOrderManagementDetail managementDetail : detailList) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
            DdlPreparedPack.getStringEqualPack(managementDetail.getPartNo(), "erpPartNo", packBean);
            DdlPreparedPack.getStringEqualPack(managementDetail.getCustPartNo(), "custPartNo", packBean);
            if(mesCustomerPartRepository.isExitByHql(packBean)){
                mesShippingOrderManagementDetail = managementDetail;
                break;
            }
        }
        if(Objects.isNull(mesShippingOrderManagementDetail)){
            throw new ImppBusiException(String.format("【%s】此条码未匹配到数据，请检查数据！", model.getSn()));
        }
        return mesShippingOrderManagementDetail;
    }


    private MesProduceSn checkMesProduceSn(MesSortShippingCheckModel model, String org) {
        //校验条码是否已经扫描
        Optional<MesShippingOrderManagementDetail> optional = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getBarcode()) && k.getBarcode().equals(model.getSn())).findFirst();
        if (optional.isPresent()) {
            throw new ImppBusiException(String.format("【%s】此条码已经扫描过，请检查数据！", model.getSn()));
        }
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getSn(), "productSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);


        if (StringUtils.isEmpty(produceSn)) {
            throw new ImppBusiException(String.format("【%s】此产品条码不存在，请检查数据！", model.getSn()));
        }
        //校验状态
        checkStatus(model, produceSn);
        return produceSn;
    }

    private void checkStatus(MesSortShippingCheckModel model, MesProduceSn produceSn) {
        //校验产品条码状态 必须是状态为已下线条码
        if (produceSn.getSnStatus() != MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue() && produceSn.getSnStatus() != MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码状态为【%s】,请选择状态为【%s】或【%s】状态的条码", model.getSn(), MesExtEnumUtil.PRODUCE_SN_STATUS.valueOfDescription(produceSn.getSnStatus()),
                    MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getDescription(),MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getDescription()));
        }
        //校验产品质量状态 必须是状态为合格条码
        if (produceSn.getQcStatus() != MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码质量状态为【%s】,请选择【%s】状态的条码", model.getSn(), MesExtEnumUtil.PRODUCE_QC_STATUS.valueOfDescription(produceSn.getQcStatus()),
                    MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getDescription()));
        }
    }

    private MesShippingOrderManagement checkMesShippingOrderManagement(MesSortShippingCheckModel model, String org) {
        //校验发运单是否存在
        DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getShippingCode(), "shippingCode", orderManagementPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderManagementPackBean);
        if (StringUtils.isEmpty(orderManagement)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在，请检查数据！", model.getShippingCode()));
        }
        return orderManagement;
    }

    public void saveDate(MesShippingOrderManagement orderManagement, MesSortShippingCheckModel model, String organizeCode, String userInfo) {
        //若发运单状态为创建或发布 则写入开始扫描时间
        if (orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() ||
                orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
            orderManagement.setStartScanTime(TimeTool.getNowTime(true));
        }

        //发运单状态修改为校验完成
        orderManagement.setSystemSyncStatus(CommonEnumUtil.FALSE);
        orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue());
        orderManagement.setEndScanTime(TimeTool.getNowTime(true));

        //发运单明细状态修改为校验完成
        DdlPackBean orderManagementDetailPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "pid", orderManagementDetailPackBean);
        List<MesShippingOrderManagementDetail> orderManagementDetailList = shippingOrderManagementDetailRepository.findByHqlWhere(orderManagementDetailPackBean);
        List<MesShippingOrderManagementDetail> detailList = model.getDetailList();
        Map<Long, List<MesShippingOrderManagementDetail>> custInfoSeqMap = detailList.stream().collect(Collectors.groupingBy(MesShippingOrderManagementDetail::getCustInfoSeq));

        List<MesProduceSn> produceSnList = new ArrayList<>();
        for (MesShippingOrderManagementDetail detail : orderManagementDetailList) {
            //根据扫描的条码查询条码是否存在
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
            if (Objects.equals(orderManagement.getIsEnableMemory(), MesCommonConstant.TRUE_INTEGER)) {
                DdlPreparedPack.getStringEqualPack(detail.getBarcode(), "productSn", packBean);
            } else {
                List<MesShippingOrderManagementDetail> mesShippingOrderManagementDetails = custInfoSeqMap.get(detail.getCustInfoSeq());
                MesShippingOrderManagementDetail managementDetail = mesShippingOrderManagementDetails.get(0);
                detail.setRemark(managementDetail.getRemark());
                detail.setStatus(managementDetail.getStatus());
                detail.setBarcode(managementDetail.getBarcode());
                DdlPreparedPack.getStringEqualPack(detail.getBarcode(), "productSn", packBean);
            }
            detail.setSystemSyncStatus(CommonEnumUtil.FALSE);
            detail.setActualQty(1);
            ConvertBean.serviceModelUpdate(detail, userInfo);
            if(!StringUtils.isEmpty(detail.getBarcode()) && !Objects.equals(orderManagement.getIsEnableMemory(), MesCommonConstant.TRUE_INTEGER)){
                DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
                MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);
                if(!Objects.isNull(produceSn)){
                    produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.SHIPPED.getValue());
                    produceSn.setSystemSyncStatus(CommonEnumUtil.FALSE);
                    ConvertBean.serviceModelUpdate(produceSn, userInfo);
                    produceSnList.add(produceSn);
                }
            }

        }

        if(!CollectionUtils.isEmpty(produceSnList)){
            produceSnRepository.saveAll(produceSnList);
        }
        update(orderManagement);
        shippingOrderManagementDetailRepository.saveAll(orderManagementDetailList);
        //1.更新装车单明细对应的发运单状态=发运
        DdlPackBean vehiclesOrderDetailPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(orderManagement.getShippingCode(), "shippingCode", vehiclesOrderDetailPackBean);
        MesLoadingListDetail vehiclesOrderDetail = vehiclesOrderDetailRepository.getByProperty(vehiclesOrderDetailPackBean);
        if(!StringUtil.isEmpty(vehiclesOrderDetail)){
            vehiclesOrderDetail.setStatus(MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
            ConvertBean.serviceModelUpdate(vehiclesOrderDetail, userInfo);
            vehiclesOrderDetailRepository.save(vehiclesOrderDetail);
        }
        if(StringUtils.isEmpty(orderManagement.getOrderCode()) || StringUtils.isEmpty(orderManagement.getLoadingListId())){
            //保存当前发运单明细报工
            jisShippingRepository.saveAll(getMesJisShippings(organizeCode, orderManagement));
        }else{
            //查询装车单明细
            DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getNumEqualPack(orderManagement.getLoadingListId(),"pid",orderManagementPackBean);
            DdlPreparedPack.getNumberSmallerEqualPack(MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.SKIP.getValue(),"status",orderManagementPackBean);
            List<MesLoadingListDetail> listDetails = vehiclesOrderDetailRepository.findByHqlWhere(orderManagementPackBean);
            if(CollectionUtils.isEmpty(listDetails)) return;
            orderManagementPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getInPackList(listDetails.stream().map(MesLoadingListDetail::getShippingCode).collect(Collectors.toList()), "shippingCode", orderManagementPackBean);
            DdlPreparedPack.getNumNOEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue(), "status", orderManagementPackBean);
            boolean exitByHql = shippingOrderManagementRepository.isExitByHql(orderManagementPackBean);
            //2.装车单中所有发运单全部变成发运回传
            if(!exitByHql){
                DdlPackBean jisBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(orderManagement.getOrderCode(), "orderCode", jisBean);
                List<MesShippingOrderManagement> shippingOrderManagementList = shippingOrderManagementRepository.findByHqlWhere(jisBean);
                for (MesShippingOrderManagement mesShippingOrderManagement : shippingOrderManagementList) {
                    //保存当前发运单明细报工
                    jisShippingRepository.saveAll(getMesJisShippings(organizeCode, mesShippingOrderManagement));
                }

                if(!StringUtil.isEmpty(vehiclesOrderDetail)){
                    MesLoadingList loadingList = vehiclesOrderRepository.getById(vehiclesOrderDetail.getPid());
                    if (!StringUtils.isEmpty(loadingList)) {
                        loadingList.setSystemSyncStatus(CommonEnumUtil.FALSE);
                        loadingList.setStatus(MesExtEnumUtil.MES_LOADING_STATUS.ASN.getValue());
                        ConvertBean.serviceModelUpdate(loadingList, userInfo);
                        shippingLoadingCheckService.update(loadingList);
                    }
                }
            }
        }

    }

    private List<MesJisShipping> getMesJisShippings(String organizeCode, MesShippingOrderManagement mesShippingOrderManagement) {
        DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(mesShippingOrderManagement.getId(), "pid", detailPackBean);
        List<MesShippingOrderManagementDetail> shippingOrderManagementDetails = shippingOrderManagementDetailRepository.findByHqlWhere(detailPackBean);
        //当前装车单所有发运单明细 需要报工的数据
        return jisShippingService.getJisShipping(mesShippingOrderManagement, shippingOrderManagementDetails, organizeCode);
    }

    @Override
    @MonitorLog
    public void update(MesShippingOrderManagement item) {
        shippingOrderManagementRepository.update(item);
    }

    @Override
    @MonitorLog
    public void updateDetail(MesShippingOrderManagementDetail item) {
        shippingOrderManagementDetailRepository.update(item);
    }

    private void updateLoadingListClose(List<Integer> loadingListStatusList,MesShippingOrderManagement orderManagement,String userName){
        List<MesShippingOrderManagement> mesShippingOrderManagementList = getMesLoadingListList(loadingListStatusList, orderManagement);
        if(CollectionUtils.isEmpty(mesShippingOrderManagementList)) return;
        List<Long> idList = mesShippingOrderManagementList.stream().map(MesShippingOrderManagement::getId).distinct().collect(Collectors.toList());
        //关闭发运单明细
        closeShippingOrderDetails(orderManagement.getOrganizeCode(), userName, idList);
        //关闭发运单
        closeShippingOrder(orderManagement.getOrganizeCode(), userName, idList);
        //更新装车单明细
        updateLoadingDetails(orderManagement.getOrganizeCode(), userName, idList);
    }

    private List<MesShippingOrderManagement> getMesLoadingListList(List<Integer> statusForCheckList, MesShippingOrderManagement orderManagement) {
        if(StringUtils.isEmpty(orderManagement.getShippingGroupCode()) || StringUtils.isEmpty(orderManagement.getSeq()))  return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(orderManagement.getOrganizeCode());
        DdlPreparedPack.getNumberSmallerPack(orderManagement.getSeq(), "seq", packBean);
        DdlPreparedPack.getStringEqualPack(orderManagement.getShippingGroupCode(),"shippingGroupCode",packBean);
        DdlPreparedPack.getInPackList(statusForCheckList, "status", packBean);
        return shippingOrderManagementRepository.findByHqlWhere(packBean);
    }

    private void closeShippingOrderDetails(String organizeCode, String userName, List<Long> idList) {
        if(CollectionUtils.isEmpty(idList) || StringUtils.isEmpty(organizeCode)) return;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(idList, "pid",packBean);
        shippingOrderManagementDetailRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime", "status","remark"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.CLOSE.getValue(), MesPcnExtConstWords.STRONGER_PASS}, packBean);
    }

    private void closeShippingOrder(String organizeCode, String userName, List<Long> idList) {
        if(CollectionUtils.isEmpty(idList)|| StringUtils.isEmpty(organizeCode)) return;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(idList, "id",packBean);
        shippingOrderManagementRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime", "status","remark"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.SHIPPING_ORDER_STATUS.CLOSE.getValue(),MesPcnExtConstWords.STRONGER_PASS}, packBean);
    }

    private void updateLoadingDetails(String organizeCode, String userName, List<Long> idList) {
        if(CollectionUtils.isEmpty(idList)|| StringUtils.isEmpty(organizeCode)) return;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(idList, "shippingId",packBean);
        vehiclesOrderDetailRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime", "status","remark"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.CLOSE.getValue(),MesPcnExtConstWords.STRONGER_PASS}, packBean);
    }
}
